-----------------------------------------------------------------------------------------------
-- Client Lua Script for ZChat
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
 
require "Window"
 
-----------------------------------------------------------------------------------------------
-- ZChat Module Definition
-----------------------------------------------------------------------------------------------
local ZChat = {} 
 
-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------

-- simple method to ge323t count of elements within array
function tlength (T) 

	local count = 0
	
	for _ in pairs(T) do count = count + 1 end
	return count
	
end

local colorTable = {
	[1]   = ApolloColor.new("ChatCommand"),
	[2]   = ApolloColor.new("ChatSystem"),
	[3]   = ApolloColor.new("ChatDebug"),
	[4]   = ApolloColor.new("ChatSay"),
	[5]   = ApolloColor.new("ChatShout"),
	[6]   = ApolloColor.new("ChatWhisper"),
	[7]   = ApolloColor.new("ChatParty"),
	[8]   = ApolloColor.new("ChatEmote"),
	[9]   = ApolloColor.new("ChatZone"),
	[10]  = ApolloColor.new("ChatPvP"),
	[11]  = ApolloColor.new("ChatTrade"),
	[12]  = ApolloColor.new("ChatGuild"),
	[13]  = ApolloColor.new("ChatGuildOfficer"),
	[14]  = ApolloColor.new("ChatCircle2"),
	[15]  = ApolloColor.new("ChatCustom"),
	[16]  = ApolloColor.new("ChatNPC"),
	[17]  = ApolloColor.new("ChatGeneral"),
	[18]  = ApolloColor.new("ChatSupport"),
	[19]  = ApolloColor.new("ChatLoot"),
	[20]  = ApolloColor.new("ChatEmote"),
	[21]  = ApolloColor.new("ChatWarParty"),
	[22]  = ApolloColor.new("ChatWarPartyOfficer"),
	[23]  = ApolloColor.new("ChatAdvice"),
	[24]  = ApolloColor.new("ChatAccountWisper")
}

local channels = {
	whisper = ChatSystemLib.ChatChannel_Whisper,
	awhisper = ChatSystemLib.ChatChannel_AccountWhisper
}
-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function ZChat:new(o)

    o = o or {}
    setmetatable(o, self)
    self.__index = self 

	self.chats = {}
	self.windows = {}
	self.chatColor = {}
	self.pendingMsg = 0
	self.unread = {}
	self.settings = {
		offset = {
			l = 300,
			t = 130
		},
		
		colors = {
			whisper = 6,
			awhisper = 24
		},
		
		chatoffset = {
			l = 405,
			t = 264,
			b = 494
		},
		
		safeConversations = true,
		showTimestamp = true,
		multiChats = false
	}

    return o

end

function ZChat:Init()	
    Apollo.RegisterAddon(self, true, "ZChat", {})
end
 

-----------------------------------------------------------------------------------------------
-- ZChat OnLoad
-----------------------------------------------------------------------------------------------

function ZChat:OnLoad()
	self.xmlDoc = XmlDoc.CreateFromFile("ZChat.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
end

-----------------------------------------------------------------------------------------------
-- ZChat OnDocLoaded
-----------------------------------------------------------------------------------------------
function ZChat:OnDocLoaded()

	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
	
	    self.wndMain = Apollo.LoadForm(self.xmlDoc, "Chats", nil, self)
	
		if self.wndMain == nil then
			Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
			return
		end

		Apollo.RegisterEventHandler("ChatMessage", "OnChatMessage", self)
		Apollo.RegisterEventHandler("ChatTellFailed", "OnChatTellFailed", self)
		Apollo.RegisterEventHandler("ChatAccountTellFailed", "OnChatTellFailed", self)
		
		self.wndMain:FindChild("ChatOverview"):Show(false)
		self.wndMain:SetAnchorOffsets(self.settings.offset.l, self.settings.offset.t, self.settings.offset.l+80, self.settings.offset.t+60)
		self:AddCount ( tlength(self.chats) )
		
	end
	
end

-----------------------------------------------------------------------------------------------
-- ZChat Functions
-----------------------------------------------------------------------------------------------

function ZChat:CreateChatWindow(sender, overwrite)

	if self.settings.multiChats then
		overwrite = true
	end

	if not self.chats[sender] or self.chats[sender] == nil then
		self.chats[sender] = {}
	end
	
	if not overwrite or overwrite == nil then
		
		if self:IsWindowOpen() then
			if not self.windows[sender] or self.windows[sender] == nil then 
				if self.unread[sender] then
					return false
				else
					self.unread[sender] = true
					self.pendingMsg = self.pendingMsg + 1
					self:AddCount()
					return false
				end
			end
		end
	
	end
	
	if self.unread[sender] then
		self.unread[sender] = false
		self.pendingMsg = self.pendingMsg - 1
		self:AddCount()
	end

	if not self.windows[sender] or self.windows[sender] == nil then
	
		local l,t,r,b = self.settings.chatoffset.l,self.settings.chatoffset.t,self.settings.chatoffset.l+365,self.settings.chatoffset.b
		
		if self:IsWindowOpen() and overwrite then
			l = l+10
			t = t+10
			b = b+10
			r = r+10
		end
	
		self.windows[sender] = Apollo.LoadForm(self.xmlDoc, "ChatWindow", nil, self)
		self.windows[sender]:SetAnchorOffsets(l,t,r,b)
				
		self.windows[sender]:FindChild("Title"):SetTextColor(colorTable[self.chatColor[sender]])
		self.windows[sender]:FindChild("Title"):SetText(sender)
		self.windows[sender]:FindChild("SendMsg"):SetTextColor(colorTable[self.chatColor[sender]])
		self.windows[sender]:FindChild("PlayerOptions"):Show(false)
				
		return true
		
	end
	
	return false

end

function ZChat:IsWindowOpen()
	for k, v in pairs ( self.windows ) do
		if self.windows[k]:IsShown() then
			return true
		end
	end
	return false
end

function ZChat:CreateChatLog(sender, window)

	window:FindChild("ChatLog"):DestroyChildren()

	for i = 0, (tlength(self.chats[sender])-1) do
	
		local chatLine = Apollo.LoadForm(self.xmlDoc, "ChatLine", window:FindChild("ChatLog"), self)		
		chatLine:SetTextColor(colorTable[self.chatColor[sender]])
		chatLine:SetText(self.chats[sender][i])
		chatLine:SetHeightToContentHeight()
		window:FindChild("ChatLog"):ArrangeChildrenVert()
		window:FindChild("ChatLog"):SetVScrollPos(window:FindChild("ChatLog"):GetVScrollRange())
				
	end

end

function ZChat:HandleNewMessage ( sender, bself, realm, msg )
	
	local bool = self:CreateChatWindow(sender)
	
	local line = tlength(self.chats[sender])
	local stamp = ""
	
	if self.settings.showTimestamp then
		local tm = GameLib.GetLocalTime()
		stamp = "[" .. string.format("%d:%02d", tm.nHour, tm.nMinute) .. "] "
	end

	if bself then 
		self.chats[sender][line] = stamp .. "Me: " .. msg
	else
		self.chats[sender][line] = stamp .. sender .. ": " .. msg
	end
	
	if bool then 
		self:CreateChatLog(sender, self.windows[sender]) 
	else
		if self.windows[sender] ~= nil then
			if self.windows[sender]:IsShown() then
				self:CreateChatLog(sender, self.windows[sender])
			end
		end
	end
		
end

-----------------------------------------------------------------------------------------------
-- ZChat Events
-----------------------------------------------------------------------------------------------

function ZChat:OnConfigure()

	self.settingsWnd = Apollo.LoadForm(self.xmlDoc, "Settings", nil, self)
	local whisperSlide = self.settingsWnd:FindChild("WhisperColor"):FindChild("SliderBar")
	local aWhisperSlide = self.settingsWnd:FindChild("AccountWhisperColor"):FindChild("SliderBar")
	local whisperWindow = self.settingsWnd:FindChild("WhisperColor"):FindChild("ColorWindow")
	local aWhisperWindow = self.settingsWnd:FindChild("AccountWhisperColor"):FindChild("ColorWindow")
	local btnSave = self.settingsWnd:FindChild("SaveChat"):FindChild("Button")
	local btnStamp = self.settingsWnd:FindChild("ShowTimestamp"):FindChild("Button")
	local btnChats = self.settingsWnd:FindChild("MultiChats"):FindChild("Button")
	
	whisperSlide:SetValue(self.settings.colors.whisper)
	aWhisperSlide:SetValue(self.settings.colors.awhisper)
	whisperWindow:SetBGColor(colorTable[self.settings.colors.whisper])
	aWhisperWindow:SetBGColor(colorTable[self.settings.colors.awhisper])
	btnSave:SetCheck(self.settings.safeConversations)
	btnStamp:SetCheck(self.settings.showTimestamp)
	btnChats:SetCheck(self.settings.multiChats)
	
end


function ZChat:OnChatMessage(channelCurrent, tMessage)

	local tChannelType = channelCurrent:GetType()
	local sender = tMessage.strSender
	local realm = tMessage.realm
	local bself = tMessage.bSelf
	local msg = tMessage.arMessageSegments[1].strText
	
	if tChannelType == ChatSystemLib.ChatChannel_Whisper then
		self.chatColor[sender] = self.settings.colors.whisper
		self:HandleNewMessage ( sender, bself, realm, msg )
	elseif tChannelType == ChatSystemLib.ChatChannel_AccountWhisper then
		self.chatColor[sender] = self.settings.colors.awhisper
		self:HandleNewMessage ( sender, bself, realm, msg )
	end
	
end

function ZChat:OnRestore(eType, tData)
	
	for key, val in pairs(self.settings) do
		if tData.settings[key] ~= nil then
			self.settings[key] = tData.settings[key]
		end
	end	
	
	if self.settings.safeConversations then
		self.chats = tData.chats
		self.chatColor = tData.chatColor
	end
	
end

function ZChat:OnSave()

	local tData = {}
	
	if self.settings.safeConversations then
		tData.chats = self.chats
		tData.chatColor = self.chatColor
	end

	self.settings.offset.l, self.settings.offset.t, _, _ = self.wndMain:GetAnchorOffsets()
	tData.settings = self.settings
	return tData
	
end

-----------------------------------------------------------------------------------------------
-- ZChatForm Functions
-----------------------------------------------------------------------------------------------

function ZChat:OnChatWindowClose(wndHandler, wndControl)

	local sender = wndControl:GetParent():FindChild("Title"):GetText()
	
	self.windows[sender] = nil
	wndControl:GetParent():GetParent():Close()
	
end

function ZChat:OnChat( wndHandler, wndControl, strText )

	local msg = wndControl:GetText()
	local player = wndControl:GetParent():FindChild("Title"):GetText()
	
	ChatSystemLib.Command(string.format("/w %s %s", player, msg))
	wndControl:SetText("")
	
end

function ZChat:OnOptions ( wndHandler, wndControl, strText )

	local l, t, r, b = wndControl:GetParent():GetParent():GetAnchorOffsets()
	local isshown = wndControl:GetParent():GetParent():FindChild("PlayerOptions"):IsShown()
	
	wndControl:GetParent():GetParent():FindChild("PlayerOptions"):Show(not isshown)	
	
	if isshown then
		wndControl:GetParent():GetParent():SetAnchorOffsets(l,t,l+365,b)
	else
		wndControl:GetParent():GetParent():SetAnchorOffsets(l,t,l+467,b)
	end
	
end

function ZChat:OnPlayerOptions ( wndHandler, wndControl, strText )

	local sender = wndControl:GetParent():GetParent():FindChild("Title"):GetText()
	
	ChatSystemLib.Command("/" .. wndControl:GetText() .. " " .. sender)
	
end

function ZChat:AddCount ( )

	self.wndMain:FindChild("Count"):SetText(self.pendingMsg)
	
end

function ZChat:OnOverview( )

	local l, t, _, __ = self.wndMain:GetAnchorOffsets()
	local ov = self.wndMain:FindChild("ChatOverview")
	local show = ov:IsShown()
	
	ov:Show(not show)
	
	if not show then
		self:FillOverview ()
	else
		self.wndMain:SetAnchorOffsets(l,t,l+80,t+60)
	end
	
end

function ZChat:FillOverview ()

	local l, t, _, __ = self.wndMain:GetAnchorOffsets()
	local ov = self.wndMain:FindChild("ChatOverview")
	
	ov:FindChild("Frame"):DestroyChildren()
		
	for k in pairs (self.chats) do
		local newWnd = Apollo.LoadForm(self.xmlDoc, "Insert", ov:FindChild("Frame"), self)
			
		if self.unread[k] then newWnd:FindChild("NameLabel"):SetTextColor(ApolloColor.new("BubbleTextRegular")) end
		newWnd:FindChild("NameLabel"):SetText(k)
		ov:FindChild("Frame"):ArrangeChildrenVert()
		ov:FindChild("Frame"):SetVScrollPos(100)
	end
		
	self.wndMain:SetAnchorOffsets(l,t,l+315,t+190)

end

function ZChat:OnOpenChatWindow( wndHandler, wndControl, eMouseButton )

	local sender = wndControl:GetParent():FindChild("NameLabel"):GetText()
	
	self:CreateChatWindow(sender, true)	
	self:CreateChatLog(sender, self.windows[sender])	
	self:FillOverview ()
	
end

---------------------------------------------------------------------------------------------------
-- Settings Functions
---------------------------------------------------------------------------------------------------


function ZChat:OnSlideChanged( wndHandler, wndControl, fNewValue, fOldValue )
	wndControl:GetParent():FindChild("ColorWindow"):SetBGColor(colorTable[fNewValue])
end

function ZChat:OnSettingsChange ( wndHandler, wndControl )

	local settingsWnd = wndControl:GetParent()
	self.settings.showTimestamp = settingsWnd:FindChild("ShowTimestamp"):FindChild("Button"):IsChecked()
	self.settings.safeConversations = settingsWnd:FindChild("SaveChat"):FindChild("Button"):IsChecked()
	self.settings.multiChats = settingsWnd:FindChild("MultiChats"):FindChild("Button"):IsChecked()
	self.settings.colors.whisper = settingsWnd:FindChild("WhisperColor"):FindChild("SliderBar"):GetValue()
	self.settings.colors.awhisper = settingsWnd:FindChild("AccountWhisperColor"):FindChild("SliderBar"):GetValue()
	
	settingsWnd:GetParent():Close()

end

function ZChat:OnSettingsClose ( wndHandler, wndControl )

	local settingsWnd = wndControl:GetParent()
	settingsWnd:GetParent():Close()
	
end

function ZChat:OnDeleteChat( wndHandler, wndControl, eMouseButton )

	local sender = wndControl:GetParent():FindChild("NameLabel"):GetText()
	
	if not self.windows[sender] or self.windows[sender] == nil then
	else
		self.windows[sender]:Close()
		self.windows[sender] = nil
	end
	
	self.chats[sender] = nil
	self.chatColor[sender] = nil
	if self.unread[sender] then 
		self.pendingMsg = self.pendingMsg - 1 
		self.unread[sender] = nil
	end
	
	self:FillOverview ()	
	self:AddCount ( )

end

---------------------------------------------------------------------------------------------------
-- ChatWindow Functions
---------------------------------------------------------------------------------------------------

function ZChat:OnChatMoved ( wndHandler, wndControl, nOldLeft, nOldTop, nOldRight, nOldBottom )
	self.settings.chatoffset.l, self.settings.chatoffset.t, _, self.settings.chatoffset.b = wndControl:GetAnchorOffsets()
end

-----------------------------------------------------------------------------------------------
-- ZChat Instance
-----------------------------------------------------------------------------------------------
local ZChatInst = ZChat:new()
ZChatInst:Init()
